<div id="messages">
{% if messages %}
{% for message in messages %}
<div class="alert alert-{{ message.tags }} {% if 'error' in message.tags %}alert-danger{% endif %} fade show">
  <button type="button" class="close" data-dismiss="alert" aria-hidden="true">&times;</button>
  {{ message }}
</div>
{% endfor %}
{% endif %}
</div>
