# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-25 19:26
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('forum_conversation', '0008_auto_20160903_0512'),
    ]

    operations = [
        migrations.AlterField(
            model_name='topic',
            name='slug',
            field=models.SlugField(max_length=255, verbose_name='Slug'),
        ),
    ]
