{% extends "admin/change_form.html" %}
{% load i18n %}

{% block object-tools-items %}
<li><a href="{% url 'admin:forum_forum_editpermission_index' object_id %}" class="permissionslink">{% trans "Forum permissions" %}</a></li>
{{ block.super }}
{% endblock %}